--[[ 

	DCS - Target Impact Tracker Script - Range Control
	by Draken35
	
	Requires MIST 4.5.106 or above <https://github.com/mrSkortch/MissionScriptingTools/tree/master>
	
    Version history
	1.00		10/04/2021	Initial release / Development start
	1.50		10/06/2021  Development Phase 2 starts
							- Target designation
							- Allow to place target definition in separated script
	1.51		11/10/2021  - Add option to silence Abort Messages
							- Designate only alive units or zones
	1.52		12/18/2021	- Fixed: not listing coordinates when there is no aimpoint	
    1.53        01/25/2022  - Added launch slant range & MOA
	1.54        01/26/2022  - Separated altitud from distance in settings
							- Added DMS & DM.mm coordinates formats
	1.55		01/27/2022	- Allow to modify weapon tracking interval
							- Added Slant range to release parameters
	1.56		01/28/2022	- Fixed MOA calculations	
							- Fixed slant range display
							- Use of MOA or MIL based on distance UOM setting
							- Settings menu redesigned
	1.57		01/30/2022	- Added intitialization on Take off and weapon add event 
	1.58		02/01/2022	- Added altitude to target coordinates report
	1.59		02/04/2022	- Changed wording on Designation menu			
	1.59.1      02/06/2022	- Replaced Mist Add message method by custom code (bug in MIST)
	1.60.01     02/11/2022  - Versioning schema changed
							- Fixed setting changes not providing feedback
							- Added Hit/Miss status in relation to closest target to each weapon
							- "Miss" message when impact is not close to any target cganged to: "No targets in impact area"
							- reduced screen footprint of release parameters information
	1.60.02     02/13/2022	- Added hit report for CIRCULAR zones
	1.61.00		02/14/2022	- Capture type of shell for strafing hits
							- added BDA report
							- Improve accuracy of weapon impact position 
	1.62.00		02/15/2022	- Added "Verbose" flag
							- Added methods to control rolling-in/attack completed mechanics externally
	1.62.01		02/15/2022	- Added data store
	                        - Added enable/Disable menu functionality
							- Fixed bug preventing listing of coordinates and designation of zones
							- Added API for data export
									- TITS.AddTarget(_TargetName,_displayName,_Type,_TrackImpact,_TrackStrafing,_ReportHits,_AllowOffsetDesignation,_AllowSmokeDesignation,_AllowLaserDesignation,_AllowListCoordinates)
									- TITS.exportTrackingData() 	: returns tracking data to the point of call 
															unitName
															targetName
															targetType
															targetDisplayName
															targetHitCount
															impactDistance      (0 if it is a shell weapon) 
															targetBearing 
															weaponID    		(-1 if it is a shell weapon)
															weaponType 
															releaseHeading 		(0 if it is a shell weapon) 
															releaseAlt 			(0 if it is a shell weapon) 
															releaseSpeed 		(0 if it is a shell weapon) 
														 	releasePitch 		(0 if it is a shell weapon) 
														 	releaseRoll 		(0 if it is a shell weapon) 
														 	releaseYaw 			(0 if it is a shell weapon) 
														 	slantRange			(0 if it is a shell weapon) 
									- TITS.resetTrackingData()	: deletes  tracking data to the point of call 
									- TITS.startAttack(unitname): reports "rolling in" for unitname
									- TITS.endAttack(unitname)	: reports "attack completed" for unitname
									- TITS.getTrackedUnitNames() : returns a table of tracked unit names
									- TITS.getTarget(_name)		: returns a row with the target description
																name, 
																displayName, 
																type, 
																impact,	
																strafing, 	
																bda,		
																des_aimoff,	
																des_wp,	
																des_laser,	
																list_coor
	1.62.02		02/17/2022		- Fixed bug affecting tracking impacts in zones in high elevations
	1.62.03		02/21/2022		- Fixed yet another bug in designation/list coords.				
								- Improved calculations of corrected weapon & moving targets positions
								- remove distance round up
								- fixed some typos
	1.62.04		02/23/2022		-add defaults to allow to display or not the Attack Results and/or 
								 Hits per Target after reporting an attack completed
								- TITS.EnterUnitEvent changed to receive "unitName" as parameter.
								 This allow to call the this method to MANUALLY initialize units
								- Attack Results report now uses the designated target(if there is anY
								  to determine if the attack was a good hit or not.
								- "Cancel laser designation" changed to "Cancel target designation"
								  This menu will now turn off laser (if on) and undesignate any target
								- For WP designation, distance to the mark will be reported in 25ft or 10m/10yd increments
	1.63.00		02/25/2022		- Added " on target!" text to indicate strafing hits on designated target
								- Added maxDeltaY for weapons position correction
								- Added ammo count track to calculate total % of rounds of target per pass (ROT)
								  (if ammo is set to unlimited it will not be calculated correctly)
	1.63.01		02/25/2022		- Added callsign to the messages
	1.63.02		02/27/2022		- Testing revealed that tracking target health to be able to calvulate distance to destroyed 
								  targets impacts too much performance. Script will support only immortal targets for now on
	1.63.03		03/29/2022		- Disabled on screen error reporting to make it a little bit more MP friendly with inmortal targets
								 

-- Ideas and plans


	- Noshoot target flag ?

	- Range areas (1 to 6)
		- JTAC per range area
		- Offset marker per range area
		- Menu option to indicate what area to use
		- commands will work only for targets in the indicated area
		
	- 2-ways range: ability to designate targets as fight-back and toggle it on/off
	
	- Toggle radars on/off to use ARMs
	
	- Talk-On Designation
		- Add description attibute to target. 
		- Add list of reference "points"
			- only zones
			- need a description attribute
		- Talk-on will used target and reference point descriptions to construct the designation message
		- ability to mark the reference point with colored smoke 


	
--]]

TITS  = {}
TITS.Version = '1.63.03'
trigger.action.outText( 'Target Impact Tracker Script- v'..TITS.Version, 2 )


--############################################################################################################################################
-- #############   Configuration section   ############# 
--############################################################################################################################################

-- These parameters can be added in a separated filed, loaded after the script and before players gets in the cockpit


TITS.MaxMissDistance 			= 1000 -- meters. Any impact over this distance from the closest target will be considered as Miss
TITS.MaxMarkDistance 			= 150 -- meters, max distance for Willie Pete (smoke) marks
TITS.HitReportDefault			= true -- default for the option to report hits in results
TITS.ReleaseParametersDefault 	= true -- default for the option to display the release parameters in results
TITS.SpeedUnitsDefault			= 1    -- 1 = Knots, 2 = Mph, 3 = Kph
TITS.DistanceUnitsDefault		= 1    -- 1 = feet, 2 = yards, 3 = meters
TITS.AltitudeUnitsDefault		= 1    -- 1 = feet, 3 = meters ( 2 is not used)
TITS.CoordinateFormatDefault	= 1	   -- 1 = MGRS, 2 = DMS, 3 = DM.mm
TITS.EnableAbortMessage			= true -- If enabled, the abort message will be displayed when releasing ordnance without notifiying range control
TITS.WeaponTrackingInterval		= 0.001 -- in second. Weapon tracking interval. This indicates how often the script will track weapons in flight.

TITS.ShowAttackResults			= true	-- if true, the Attack Results report will be shown automatically on "Off - Attack completed"
TITS.ShowHitsPerTarget			= false -- if true, the Hits per Target report will be shown automatically on "Off - Attack completed"

TITS.Verbose					= true  -- When false, it will disable the acknolegment messages and automatic result reporting after "attack completed"
TITS.MenuEnabled				= true	-- When false, the Range menu's will not be added to any unit
TITS.StoreData					= false -- When TRUE the data export capability via API will return data

TITS.maxDeltaY					= 15	-- in meters. Max diff on altitude between target and weapon that will trigger the position correction

TITS.LaserDesignatorName = '' -- Has to be an unit (unitname)
TITS.LaserDesignatorCode = ''
TITS.OffsetMarkerName = '' -- Offset Marker name -Has to be an unit (unitname). Offset bearing and range will be calculated from this unit

-- end parameters

-- Target list

TITS.TargetList = {} -- These are the targets that the script handles 
					 -- for the unit, use the UNIT name, not the group.
					 -- Only units can be strafed and have a bda (hit)
					 -- target names need to be UNIQUE. It is also case sensitive.
					 -- impact = true will consider the target when calculating the closest target to the weapon impact. If false it will not be considered.
					 -- strafing = true will allow the target to count strafing hits. If false, it will not
					 -- bda = true will make the script check if the target was in the weapon blast radio and was "hit" by the blast.  If false, it will not
					 -- The existance or type of the targets IS NOT VALIDATED by the script to save some CPU cycles. Make sure the list match the mission
			
	
	--table.insert(TITS.TargetList,	{	name = 'target', 		type = 'unit', impact =true,	strafing = false, 	bda = false,		des_aimoff = false,	des_wp = false,	des_laser = true,	list_coor = true})
	--table.insert(TITS.TargetList,	{	name = 'zone',			type = 'zone', impact =true,	strafing = false, 	bda = false,		des_aimoff = false,	des_wp = false,	des_laser = false,	list_coor = true})
	
	
--############################################################################################################################################
-- #############   Sound Files     ############# 
--############################################################################################################################################
	TITS.messageBeep = '204521__redoper__roger-beep.ogg' -- leave empty for no sound



--############################################################################################################################################
-- #############   Code section   ############# 
--############################################################################################################################################
-- Initialization of globals

TITS.eventHandler = {}
TITS.menuAddedToGroup = {}
TITS.attackingUnits = {}
TITS.isStrafingAttack = {}
TITS.reportWeaponsDistance = {} -- report non-strafing hits
TITS.reportWeaponsHits = {} -- report non-strafing hits
TITS.strafingHitCounter = {}
TITS.weaponsHit = {}
TITS.rootPath = {}
TITS.rollInPath = {}
TITS.offPath = {}
TITS.lastAbortMSG = {}
TITS.passCounter = {}
TITS.SpeedUOM ={} 
TITS.DistanceUOM ={} 
TITS.AltitudeUOM ={} 
TITS.CoordFormat ={} 
TITS.DisplayDBA = {}
TITS.DisplayReleaseData = {}
TITS.LastDesignationMsg = {}
TITS.designatedTargetName = '' 
TITS.LaserSpot = nil
TITS.IRSpot = nil

TITS.lastOSclock = timer.getTime()
TITS.avgTrackTic = 0

TITS.dataStore_hits = {}
TITS.dataStore_distance = {}

TITS.shellAmmoCounter = {}

--############################################################################################################################################
function TITS.eventHandler:onEvent(_eventDCS)

    if _eventDCS == nil or _eventDCS.initiator == nil then
        return true
    end

    local status, err = pcall(function(_event)

	--trigger.action.outText( string.format("Event id %s", _event.id ), 30 )
	-- ### client entered unit   _event.id == 15   
    if _event.id == world.event.S_EVENT_BIRTH  or _event.id == world.event.S_EVENT_TAKEOFF or _event.id == world.event.S_EVENT_ENGINE_STARTUP  then  -- id == 15 , 3 ,18

		TITS.EnterUnitEvent(_event.initiator:getName())
			
	elseif (_event.id == world.event.S_EVENT_SHOT) and _event.initiator:getPlayerName()  then -- id == 1 Shot
		
			TITS.ShotEvent(_event)

	 elseif  _event.id == world.event.S_EVENT_HIT and _event.initiator:getPlayerName()  then  -- id == 2 Hit
	 
			TITS.HitEvent(_event)
	
	 elseif  _event.id == world.event.S_EVENT_SHOOTING_START and _event.initiator:getPlayerName()  then   -- id == 23 Start Shooting
	 
		TITS.StartShootingEvent(_event)

	end -- if _event.id == S_EVENT_PLAYER_ENTER_UNIT then --player entered unit
  -- ### End event processing

        return true
    end, _eventDCS)

    if (not status) then 
		local msg = string.format("Target Impact Tracker (v%s): Error while handling event %s",TITS.Version, err)
        env.error(msg,false)
	--	trigger.action.outText( msg, 30 )
    end
end
--############################################################################################################################################
function TITS.EnterUnitEvent(unitName)
	local unit = Unit.getByName(unitName)
	local groupID = unit:getGroup():getID()

	--function TITS.EnterUnitEvent(_event)
	--local groupID = _event.initiator:getGroup():getID()
	--local unitName = _event.initiator:getName()
	
	--if not TITS.menuAddedToGroup[groupID] then
	-- inits
	TITS.attackingUnits[unitName] = false
	TITS.reportWeaponsHits[unitName] = false
	TITS.lastAbortMSG[unitName] = timer.getTime() - 5
	TITS.passCounter[unitName] = 0
	TITS.SpeedUOM[unitName] = TITS.SpeedUnitsDefault	
	TITS.DistanceUOM[unitName] = TITS.DistanceUnitsDefault	
	TITS.AltitudeUOM[unitName] = TITS.AltitudeUnitsDefault
	TITS.CoordFormat[unitName] = TITS.CoordinateFormatDefault
	
	TITS.DisplayDBA[unitName] = TITS.HitReportDefault
	TITS.DisplayReleaseData[unitName] = TITS.ReleaseParametersDefault
	TITS.LastDesignationMsg[unitName] = 'no targets designated'
	
	TITS.dataStore_distance[unitName] = {}
	TITS.dataStore_hits[unitName] = {}
	
	TITS.shellAmmoCounter[unitName] = {}
	TITS.targetHealth = {}
	TITS.targetHealth0 = {}
	
	TITS.CreateMenu(unitName)

		
	--end -- if not TITS.menuAddedToGroup[groupID] then
	return true
end
--############################################################################################################################################
function TITS.CreateMenu(unitName)
	local unit = Unit.getByName(unitName)
	local groupID = unit:getGroup():getID()
	
	
	if TITS.MenuEnabled and not TITS.menuAddedToGroup[groupID] then
	local rangeMenu = "Range" --string.format("Range (%s)", unitName)
		TITS.rootPath[unitName] = missionCommands.addSubMenuForGroup(groupID, rangeMenu)			
		local _reportsPath = missionCommands.addSubMenuForGroup(groupID, "Reports", TITS.rootPath[unitName])
			missionCommands.addCommandForGroup(groupID,"List coordinates"		, _reportsPath, TITS.showTargetsCoordinates		, unitName)
			missionCommands.addCommandForGroup(groupID,"Attack results"			, _reportsPath, TITS.ReportAttackResults		, unitName)
			missionCommands.addCommandForGroup(groupID,"BDA"					, _reportsPath, TITS.WeaponHitsTargetReport		, unitName)
		
		local _designationPath = missionCommands.addSubMenuForGroup(groupID, "Designation"	, TITS.rootPath[unitName])
			missionCommands.addCommandForGroup(groupID,"Mark target (WP)"					, _designationPath, TITS.MarkTargetWP			, unitName)
			missionCommands.addCommandForGroup(groupID,"Offset designation"					, _designationPath, TITS.OffsetDesignation		, unitName)
			missionCommands.addCommandForGroup(groupID,"Laser/IR designation"				, _designationPath, TITS.LaserDesignation		, unitName)	
			missionCommands.addCommandForGroup(groupID,"Cancel target designation"			, _designationPath, TITS.CancelLaserDes			, unitName)	
			missionCommands.addCommandForGroup(groupID,"Repeat last designation message"	, _designationPath, TITS.RepeatLastDesMSG		, unitName)
		
		local _settingsPath = missionCommands.addSubMenuForGroup(groupID, "Settings", TITS.rootPath[unitName])
			missionCommands.addCommandForGroup(groupID,"Display current settings"				, _settingsPath, TITS.Settings			, {unitName,0}) 
			missionCommands.addCommandForGroup(groupID,"Toggle non strafing hits report"	, _settingsPath, TITS.Settings			, {unitName,1}) 
			missionCommands.addCommandForGroup(groupID,"Toggle display of Release data"			, _settingsPath, TITS.Settings			, {unitName,2}) 
			local _settings1Path = missionCommands.addSubMenuForGroup(groupID, "Distance units", _settingsPath) 
				missionCommands.addCommandForGroup(groupID,"Distance: set to meters"				, _settings1Path, TITS.Settings		, {unitName,3}) 
				missionCommands.addCommandForGroup(groupID,"Distance: set to feet"					, _settings1Path, TITS.Settings		, {unitName,4}) 
				missionCommands.addCommandForGroup(groupID,"Distance: set to yards"					, _settings1Path, TITS.Settings		, {unitName,5}) 
			local _settings2Path = missionCommands.addSubMenuForGroup(groupID, "Altitude units", _settingsPath) 
				missionCommands.addCommandForGroup(groupID,"Altitude: set to meters"				, _settings2Path, TITS.Settings		, {unitName,6}) 
				missionCommands.addCommandForGroup(groupID,"Altitude: set to feet"					, _settings2Path, TITS.Settings		, {unitName,7}) 		
			local _settings3Path = missionCommands.addSubMenuForGroup(groupID, "Speed units", _settingsPath) 
				missionCommands.addCommandForGroup(groupID,"Speed: set to kph"						, _settings3Path, TITS.Settings		, {unitName,8}) 
				missionCommands.addCommandForGroup(groupID,"Speed: set to knots"					, _settings3Path, TITS.Settings		, {unitName,9}) 
				missionCommands.addCommandForGroup(groupID,"Speed: set to mph"						, _settings3Path, TITS.Settings		, {unitName,10}) 	
			local _settings4Path = missionCommands.addSubMenuForGroup(groupID, "Coordinates format", _settingsPath) 
				missionCommands.addCommandForGroup(groupID,"Coord. format set to MGRS"				, _settings4Path, TITS.Settings		, {unitName,11}) 
				missionCommands.addCommandForGroup(groupID,"Coord. format set to DMS"				, _settings4Path, TITS.Settings		, {unitName,12}) 
				missionCommands.addCommandForGroup(groupID,"Coord. format set to DM.mm"				, _settings4Path, TITS.Settings		, {unitName,13}) 
	
		TITS.rollInPath[unitName] = missionCommands.addCommandForGroup(groupID,"Rolling in"			, TITS.rootPath[unitName], TITS.ReportAttack		, {unitName,'in'})
		TITS.menuAddedToGroup[groupID] = true
	end -- if not TITS.menuAddedToGroup[groupID] then
	return true
end
--############################################################################################################################################
function TITS.toggleAttackMenu(unitName, action)
	local unit = Unit.getByName(unitName)
	local groupID = unit:getGroup():getID()
	if TITS.MenuEnabled then
	-- in

		if action == 'in' then

			missionCommands.removeItemForGroup(groupID, TITS.rollInPath[unitName])
			TITS.offPath[unitName] =  missionCommands.addCommandForGroup(groupID,"Off - attack completed"	,  TITS.rootPath[unitName], TITS.ReportAttack		, {unitName,'off'})		
		else
	--off
			missionCommands.removeItemForGroup(groupID, TITS.offPath[unitName])
			TITS.rollInPath[unitName] = missionCommands.addCommandForGroup(groupID,"Rolling in"				,  TITS.rootPath[unitName], TITS.ReportAttack		, {unitName,'in'})
		end -- if action...
	end -- if TITS.MenuEnabled then
	return true
end
--############################################################################################################################################
function TITS.ShotEvent(_event)
	local _weapon = _event.weapon
	local _target = _weapon:getTarget() 
	local unitName = _event.initiator:getName()
	
	if not TITS.attackingUnits[unitName] then
		local t =  timer.getTime() 
		if (t - TITS.lastAbortMSG[unitName]) > 5 then 
			if TITS.EnableAbortMessage	and TITS.Verbose then
				TITS.messageToUnit(unitName,'Abort! Abort! Abort!. You are not cleared to employ weapons.',{},10)
			end
			TITS.lastAbortMSG[unitName] = t
		end
	else
		local params = {}
		params.unitName = _event.initiator:getName()
		params.releaseHeading =  mist.utils.toDegree(mist.getHeading(_event.initiator)) -- in degrees. Players release heading
		local pos = _event.initiator:getPosition().p
		local height =  land.getHeight({x = pos.x, y = pos.z})
		params.launchPos = pos
		params.releaseAlt = pos.y - height -- in meters. Release AGL
		params.releasePitch =  mist.utils.toDegree(mist.getClimbAngle(_event.initiator)) -- in degrees. Release dive angle
		params.releaseRoll =  mist.utils.toDegree(mist.getRoll(_event.initiator)) -- in degrees. 
		params.releaseYaw =  mist.utils.toDegree(mist.getYaw(_event.initiator)) -- in degrees. 
		params.releaseSpeed = mist.vec.mag(_event.initiator:getVelocity()) -- meters/second
		params.weapon = _weapon
		params.weaponName = _weapon:getTypeName()
		params.weaponID = _weapon:getName()
		params.weaponLastPoint = _weapon:getPoint()
		params.weaponLastVelocity = _weapon:getVelocity()
	
		TITS.lastOSclock = timer.getTime()
		TITS.avgTrackTic = 0
		timer.scheduleFunction(TITS.TrackWeapon, params,  timer.getTime() +  TITS.WeaponTrackingInterval ) 					
	end
	return true
end
--############################################################################################################################################
function TITS.StartShootingEvent(_event)
	local unitName = _event.initiator:getName()


	if not TITS.attackingUnits[unitName] then
		local t =  timer.getTime() 
		if (t - TITS.lastAbortMSG[unitName]) > 5 then 
			if TITS.EnableAbortMessage	and TITS.Verbose then
				TITS.messageToUnit(unitName,'Abort! Abort! Abort!. You are not cleared to employ weapons.',{},10)
			end
			TITS.lastAbortMSG[unitName] = t
		end
	else
		TITS.isStrafingAttack[unitName] = true
	end
	return true
end
--############################################################################################################################################
function TITS.HitEvent(_event)
	local unitName = _event.initiator:getName()
	local eventTarget = _event.target 
	local shellType = _event.weapon:getTypeName()
	local isStrafingWeapon = string.find(_event.weapon:getTypeName():lower(), "weapons.shell")
	local targetName = ''
	
	if eventTarget then
		targetName = eventTarget:getName()
	end
	
	
	if TITS.attackingUnits[unitName] and targetName ~= '' then	
		if isStrafingWeapon then 
			local list = TITS.strafingHitCounter[unitName]
			if list ~= nil then
				for t,v in pairs(list) do
					if t == targetName then
						if TITS.strafingHitCounter[unitName][targetName] == nil then
							TITS.strafingHitCounter[unitName][targetName] = {}
						end -- if TITS.strafingHitCounter[unitName][targetName] == nil then
							
						if TITS.strafingHitCounter[unitName][targetName][shellType] == nil then
							TITS.strafingHitCounter[unitName][targetName][shellType] = 0
						end -- if TITS.strafingHitCounter[unitName][targetName][shellType] == nil then
							
						TITS.strafingHitCounter[unitName][targetName][shellType] = TITS.strafingHitCounter[unitName][targetName][shellType] + 1
						break
					end
				end -- for t,v in pairs(list) do
			end -- if list ~= nil then
		else -- it goes big boom	
			local target = TITS.getTarget(targetName)
			if target then
				if target.bda then
					-- record the target that got splashed by the weapon BDA
					local r = {
									weaponID = _event.weapon:getName()
								,	weaponType = _event.weapon:getTypeName()
								,	Target = targetName
							}				
					table.insert(TITS.reportWeaponsHits[unitName],r)
					if TITS.StoreData then
						local r = {
									weaponID = _event.weapon:getName()
								,	weaponType = _event.weapon:getTypeName()
								,	Target = targetName
								, 	Count = 1
							}	
						table.insert(TITS.dataStore_hits[unitName],r)
					end

				end
			end
		end -- if isStrafingWeapon then 
	end
	
	return true
end
--############################################################################################################################################
function TITS.ReportAttack(_args)
	local unitName = _args[1]
	local value = _args[2]
	--local groupID = Unit.getByName(unitName):getGroup():getID()
	local unit = Unit.getByName(unitName)
	local groupID = unit:getGroup():getID()

	TITS.passCounter[unitName] = TITS.passCounter[unitName] + 1

	TITS.attackingUnits[unitName] = (value == 'in')
	if TITS.attackingUnits[unitName] then
		if TITS.Verbose	then
			TITS.messageToUnit(unitName,'Roger. Cleared in hot',{},10)
		end
		-- reset prior attack results
		TITS.ResetStrafingHitCounter(unitName)		
		TITS.reportWeaponsDistance[unitName] = {}
		TITS.reportWeaponsHits[unitName] = {}
		TITS.toggleAttackMenu(unitName, "in")
		--missionCommands.removeItemForGroup(groupID, TITS.rollInPath[unitName])
		--TITS.offPath[unitName] =  missionCommands.addCommandForGroup(groupID,"Off - attack completed"	,  TITS.rootPath[unitName], TITS.ReportAttack		, {unitName,'off'})		
		-- Ammo counter
		local ammoTable = unit:getAmmo()
		for _,r in pairs (ammoTable) do
			local t = r.desc
			local c = r.count
			--trigger.action.outText(mist.utils.tableShow(t),25)
			if string.find(t.typeName:lower(), "weapons.shell") then
				if TITS.shellAmmoCounter[unitName][t.typeName] == nil then
					TITS.shellAmmoCounter[unitName][t.typeName] = 0
				end
				TITS.shellAmmoCounter[unitName][t.typeName] = c
			--	trigger.action.outText(t.typeName..' -> '..c..' rounds', 25)
			end
		end -- for _,r in pairs (ammoTable) do
		-- get target health 
		for t,v in pairs(TITS.TargetList) do
		--if TITS.IsAlive(v.name) and v.type == 'unit' and v.bda then
		if v.type == 'unit' and v.bda then
			if TITS.targetHealth[v.name] == nil then
				TITS.targetHealth[v.name] = 0
				TITS.targetHealth0[v.name] =  Unit.getByName(v.name):getLife0()
			end
			TITS.targetHealth[v.name] = Unit.getByName(v.name):getLife()
			
		end
	end
	else
		if TITS.Verbose	then
			TITS.messageToUnit(unitName,'Roger. Standby for report',{},10)
			
			if TITS.ShowAttackResults then		
				TITS.ReportAttackResults(unitName)
			end
			if TITS.ShowHitsPerTarget then
				TITS.WeaponHitsTargetReport(unitName)
			end
			
		end
		-- report shell hits
		TITS.storeStrafingHits(unitName)
		TITS.toggleAttackMenu(unitName, "off")
		--missionCommands.removeItemForGroup(groupID, TITS.offPath[unitName])
		--TITS.rollInPath[unitName] = missionCommands.addCommandForGroup(groupID,"Rolling in"				,  TITS.rootPath[unitName], TITS.ReportAttack		, {unitName,'in'})
	end
	return true
end
--############################################################################################################################################
function TITS.ReportAttackResults(_unitName)
	local unit = Unit.getByName(_unitName)
	local player = unit:getPlayerName()	
	local dist_uom 
	
	-- Strafing results
	-- get shell ammo left
	local ammoTable = unit:getAmmo()
	local ammoLeft = 0
	for _,r in pairs (ammoTable) do
		local t = r.desc
		local c = r.count
		--trigger.action.outText(mist.utils.tableShow(t),25)
		if string.find(t.typeName:lower(), "weapons.shell") then
			ammoLeft = ammoLeft + c
		--	trigger.action.outText(t.typeName..' -> '..c..' rounds', 25)
		end
	end -- for _,r in pairs (ammoTable) do
		
	
	
	local strafingResults = ''
	local unitTargetList = TITS.strafingHitCounter[_unitName]
	local totalHits = 0
	if TITS.strafingHitCounter[_unitName] ~= nil then
		for t,shellList in pairs(unitTargetList) do
			if shellList ~= nil  then
				local target = TITS.getTarget(t)
				if target then
					local tot = 0
					for s,count in pairs(shellList) do
						tot = tot + count
					end -- for s,count in pairs(shellList)
					if tot > 0 then
						local q = ''
						totalHits = totalHits + tot
						if (t == TITS.designatedTargetName) then 
							q = 'on target!'
						end
						strafingResults = strafingResults..string.format("\n%s,  %i hit(s) %s",target.displayName,tot, q)
					end
				end -- if target then
			end -- if shellList > 0 then
		end -- for t,shellList in pairs(unitTargetList) do
	end
	
	if TITS.isStrafingAttack[_unitName] then
		if strafingResults == '' then
			strafingResults = 'no targets hit'
		else
			local roundsFired = 0
			local t = TITS.shellAmmoCounter[_unitName]
			for s,_ in pairs (t) do
				 roundsFired = roundsFired + TITS.shellAmmoCounter[_unitName][s]
			end
			roundsFired = roundsFired - ammoLeft
			local eff = math.floor(100*totalHits/roundsFired)
			strafingResults = string.format('\n  ROT:%s%%  %s',eff,strafingResults)
		end
		strafingResults = 'Strafing attack: '.. strafingResults
	end 
	
-- Impact distace results and BDA
	
	local distanceResult = ''
	
	for _,r in pairs(TITS.reportWeaponsDistance[_unitName]) do
		local impact = 'No targets in impact area'
		local targetsHit = ''
		local HITStatus = 'Miss'
		
		for _,h in pairs(TITS.reportWeaponsHits[_unitName]) do
			if h.weaponID == r.weaponID then
				if (TITS.designatedTargetName == '') or (r.closestTarget == TITS.designatedTargetName) then 
					HITStatus = 'Good Hit!'
				else
					HITStatus = 'Wrong target hit.'
				end
			end
		end -- for _,h in pairs(TITS.reportWeaponsHits[_unitName]) do

		
		if r.impactDistance <= TITS.MaxMissDistance  then
			local t = TITS.getTarget(r.closestTarget)
			local d,duom = TITS.convertDistance(_unitName,r.impactDistance)
			-- 1 MOA = 0.290888 mil
			local AngM = r.impactDistanceMOA
			local AngMuom = 'moa'
			if duom == 'm' then
				AngM = r.impactDistanceMOA*0.290888
				AngMuom = 'mil'
			end
			
			AngM = math.floor((AngM+0.005) * 100 )/100
			if r.corrected == 'c' then
				duom = duom..'.'
			end
			impact = string.format("%s %s of %s @ %s o'clock (%s %s) <%s>"
					, math.floor(d) 
					, duom
					, t.displayName
					, r.impactDirection 
					, AngM
					, AngMuom
					, HITStatus
					
			)
		end 
		------ BDA
		local targetsHit = ''
		local targetsHitDisplay = ''
		for _,h in pairs(TITS.reportWeaponsHits[_unitName]) do
			if h.weaponID == r.weaponID then
				if not string.find(targetsHit, h.Target) then
					t = TITS.getTarget(h.Target)
					if targetsHit ~= '' then
						targetsHit = targetsHit..', '
						targetsHitDisplay = targetsHitDisplay..', '
					end
					targetsHit = targetsHit..h.Target
					targetsHitDisplay = targetsHitDisplay..t.displayName
				end			
			end			 			
		end
		if targetsHit == '' then
			targetsHit = 'none'
		end
		
		local tempStr = string.format( ">>> %s   : %s"								   
						, r.weaponType 
						, impact
					)
		if TITS.DisplayDBA[_unitName] then
			tempStr = string.format("%s\n     Targets hit: %s",tempStr,targetsHitDisplay)
		end
		
		
		-- Release parameters
		local alt, alt_uom = TITS.convertAltitude(_unitName,r.releaseAlt)
		alt = math.floor(alt+0.5)
		local Speed,speed_uom =TITS.convertSpeed(_unitName,r.releaseSpeed)
		Speed = math.floor(Speed+0.5)
		
		local sr,sruom =TITS.convertDistance(_unitName,r.slantRange)

		sr =  math.floor(sr)
		local Pitch = string.format("%s°",math.floor((r.releasePitch+0.005) *100 )/100)
		local Roll = string.format("%s°",math.floor((r.releaseRoll +0.005) *100 )/100)
		local Yaw = string.format("%s°",math.floor((r.releaseYaw  +0.005) *100 )/100)


		local relP = string.format("     Rel. Params:     Alt: %s%s AGL     Speed: %s%s\n     S.RNG: %s%s      P: %s  R:%s  Y:%s"
								   , alt , alt_uom
								   , Speed , speed_uom
								   , sr , sruom
								   , Pitch
								   , Roll
								   , Yaw
								  )
		
		if TITS.DisplayReleaseData[_unitName] then
			tempStr = string.format("%s\n%s",tempStr,relP)
		end
	
		if tempStr ~= '' then
			distanceResult = distanceResult..'\n'..tempStr
		end
	end	-- for _,r in pairs(TITS.reportWeaponsDistance[_unitName]) do

	local results = ''
	if strafingResults ~= '' then
		results = strafingResults..'\n\n'
	end
	if distanceResult ~= '' then
		results = results..distanceResult..'\n\n'
	end

	TITS.messageToUnit(_unitName,results,{},30)
	return true
end
--############################################################################################################################################
function TITS.WeaponHitsTargetReport(_unitName)
	local totals = {}
	
	-- add straffing results
	local unitTargetList = TITS.strafingHitCounter[_unitName]
	if unitTargetList  ~= nil then
		for t,shellList in pairs(unitTargetList) do
			if shellList ~= nil  then						
				for s,count in pairs(shellList) do
					totals[t] = {}
					totals[t][s] = count
				end -- for s,count in pairs(shellList)
			end -- if shellList > 0 then
		end -- for t,shellList in pairs(unitTargetList) do
	end
	-- add attack results
	unitTargetList = TITS.reportWeaponsHits[_unitName]
	if unitTargetList  ~= nil then
		for _,row in pairs(unitTargetList) do
			if totals[row.Target] == nil then
				totals[row.Target] = {}
			end
			if totals[row.Target][row.weaponType] == nil then
				totals[row.Target][row.weaponType] = 0
			end
			totals[row.Target][row.weaponType] = totals[row.Target][row.weaponType] + 1 
		end -- for _,row in pairs(unitTargetList) do
	end
	-- display the message
	local msg = 'Attack pass BDA'
	for t,wl in pairs(totals) do
		local target = TITS.getTarget(t)
		local dmg = 0
		if target.type == 'unit' and target.bda then
			--local currentLife =  0
			--if TITS.IsAlive(t) then
			--	currentLife = Unit.getByName(t):getLife()
			--else
			--	currentLife =  0
			--end
			--dmg = math.floor(((TITS.targetHealth[t]-currentLife)*100/TITS.targetHealth0[t])+0.5)
		end
		
		--msg = string.format('%s\n>> %s  (%s%% dmg)', msg, target.displayName, dmg)
		msg = string.format('%s\n>> %s', msg, target.displayName)
		for w,h in pairs(wl) do
			msg = string.format('%s\n    - %-20s: %s hits', msg,w:gsub("weapons.shells.", ""),h)
		end -- for w,h in pairs(wl) do
	end -- for t,w in pairs(totals) do
	
	
	TITS.messageToUnit(_unitName,msg,{},30)

end
--############################################################################################################################################
function TITS.messageToUnit(_unitName,_messageText,_soundTable,_displayTime)
	--[[
	local msg = {} 
	msg.text = _messageText		
	msg.msgFor = {units = {_unitName}}
	
	if TITS.messageBeep ~= '' then
		msg.sound = TITS.messageBeep 
	end
	
	if _soundTable ~= nil then
		msg.multSound  = _soundTable
	end
	
	if _displayTime == nil then
		_displayTime = 5
	end 

	msg.displayTime = _displayTime
	
	mist.message.add(msg)
	--]]
 
	-- bypass mist because of message history spamming
	if _displayTime == nil then
		_displayTime = 5
	end 
	local _unit = Unit.getByName(_unitName)
	local _groupId = _unit:getGroup():getID()
	local _callSign = _unit:getCallsign()
	if TITS.messageBeep ~= '' then
		trigger.action.outSoundForGroup(_groupId,TITS.messageBeep)
	end
	local _msg = string.format('(%s): %s',_callSign,_messageText)
	trigger.action.outTextForGroup(_groupId, _msg, _displayTime)
	
	return true
end
--############################################################################################################################################
function TITS.Settings(_args)
	local unitName = _args[1]
	local setting = _args[2]


	if setting ==  0 then -- Display settings
		local _msg  = ''
		
		if TITS.DisplayDBA[unitName] then
			 _msg = 'Non-strafing hits (BDA) reports is ON'
		else
			_msg = 'Non-strafing hits (BDA) reports is OFF'
		end
		
		if TITS.DisplayReleaseData[unitName] then
			_msg = string.format ('%s\nRelease data display is ON',_msg)
		else
			_msg = string.format ('%s\nRelease data display is OFF',_msg)
		end	
		
		if TITS.DistanceUOM[unitName] == 3 then -- Distance meters
			_msg = string.format ('%s\nDistance: Using meters',_msg)
		elseif TITS.DistanceUOM[unitName] == 2 then --Distance  feet
			_msg = string.format ('%s\nDistance: Using feet',_msg)
		else -- Distance yards
			_msg = string.format ('%s\nDistance: Using yards',_msg)
		end 
		
		if TITS.AltitudeUOM[unitName] == 3  then -- Altitude meters 
			_msg = string.format ('%s\nAltitude: Using meters',_msg)	
		else
			_msg = string.format ('%s\nAltitude: Using feet',_msg)	
		end
		
		if TITS.SpeedUOM[unitName] == 3 then -- Speed kph
			_msg = string.format ('%s\nSpeed: Using kph',_msg)
		elseif TITS.SpeedUOM[unitName] == 1  then -- Speed kt
			_msg = string.format ('%s\nSpeed: Using knots',_msg)
		else -- Speed mph
			_msg = string.format ('%s\nSpeed: Using mph',_msg)		
		end
		
		if TITS.CoordFormat[unitName] == 1  then -- Coordinate format MGRS	
			_msg = string.format ('%s\nCoordinate format: Using MGRS',_msg)
		elseif TITS.CoordFormat[unitName] == 2 then -- Coordinate format DMS	
			_msg = string.format ('%s\nCoordinate format: Using DMS',_msg)	
		else -- Coordinate format DM.mm		
			_msg = string.format ('%s\nCoordinate format: Using DM.mm',_msg)			
		end 
	
		TITS.messageToUnit(unitName,_msg,{},10)
	
	elseif setting ==  1 then -- Toggle non strafing hits report
		TITS.DisplayDBA[unitName] = not TITS.DisplayDBA[unitName] 		
		if TITS.DisplayDBA[unitName] then
			TITS.messageToUnit(unitName,'Non-strafing hits (BDA) reports is ON')
		else
			TITS.messageToUnit(unitName,'Non-strafing hits (BDA) reports is OFF')
		end
	elseif setting == 2 then -- Toggle display of Release data
		TITS.DisplayReleaseData[unitName] = not TITS.DisplayReleaseData[unitName] 		
		if TITS.DisplayReleaseData[unitName] then
			TITS.messageToUnit(unitName,'Release data display is ON')
		else
			TITS.messageToUnit(unitName,'Release data display is OFF')
		end	
	-- Distancce
	elseif setting == 3 then -- Distance meters
		TITS.DistanceUOM[unitName] = 3 -- 1 = feet, 2 = yards, 3 = meters
		TITS.messageToUnit(unitName,'Distance: Using meters')
	elseif setting == 4 then --Distance  feet
		TITS.DistanceUOM[unitName] = 1 -- 1 = feet, 2 = yards, 3 = meters
		TITS.messageToUnit(unitName,'Distance: Using feet')
	elseif setting == 5 then -- Distance yards
		TITS.DistanceUOM[unitName] = 2 --1 = feet, 2 = yards, 3 = meters
		TITS.messageToUnit(unitName,'Distance: Using yards')
	-- Altitude
	elseif setting == 6 then -- Altitude meters 
		TITS.AltitudeUOM[unitName] = 3 -- 1 = feet,  3 = meters
		TITS.messageToUnit(unitName,'Altitude: Using meters')	
	elseif setting == 7 then -- Altitude feet 
		TITS.AltitudeUOM[unitName] = 1 -- 1 = feet,  3 = meters
		TITS.messageToUnit(unitName,'Altitude: Using feet')	
	-- Speed
	elseif setting == 8 then -- Speed kph
		TITS.SpeedUOM[unitName] = 3 -- 1 = Knots, 2 = Mph, 3 = Kph	
		TITS.messageToUnit(unitName,'Speed: Using kph')
	elseif setting == 9 then -- Speed kt
		TITS.SpeedUOM[unitName] = 1 -- 1 = Knots, 2 = Mph, 3 = Kph	
		TITS.messageToUnit(unitName,'Speed: Using knots')
	elseif setting == 10 then -- Speed mph
		TITS.SpeedUOM[unitName] = 2 -- 1 = Knots, 2 = Mph, 3 = Kph	
		TITS.messageToUnit(unitName,'Speed: Using mph')		
	-- Coordinate format
	elseif setting == 11 then -- Coordinate format MGRS	
		TITS.CoordFormat[unitName] = 1 -- 1 = MGRS, 2 = DMS, 3 = DM.mm	
		TITS.messageToUnit(unitName,'Coordinate format: Using MGRS')
	elseif setting == 12 then -- Coordinate format DMS	
		TITS.CoordFormat[unitName] = 2 -- 1 = MGRS, 2 = DMS, 3 = DM.mm	
		TITS.messageToUnit(unitName,'Coordinate format: Using DMS')	
	elseif setting == 13 then -- Coordinate format DM.mm		
		TITS.CoordFormat[unitName] = 3 -- 1 = MGRS, 2 = DMS, 3 = DM.mm	
		TITS.messageToUnit(unitName,'Coordinate format: Using DM.mm')			
	end -- main if		
		

	return true
end
--############################################################################################################################################
function TITS.ResetStrafingHitCounter(_unitName)

	TITS.isStrafingAttack[_unitName] = false
	for _,t in pairs(TITS.TargetList) do
		if t.strafing and t.type ==  'unit' then
			u = Unit.getByName(t.name)
			if u then
				if  TITS.strafingHitCounter[_unitName] == nil then
					TITS.strafingHitCounter[_unitName] = {}
				end
				TITS.strafingHitCounter[_unitName][t.name] = {}
			-- else ignore it, doesn't exists
			end -- if u then
		end -- if t.strafing and type ==  'unit'then
		
	end

	return true
end
--############################################################################################################################################
function TITS.getTarget(_name)
	for _,t in pairs(TITS.TargetList) do
		if t.name  == _name then
			return t
		end
	end
	return nil
end
--############################################################################################################################################
function TITS.TrackWeapon(params,time)	
	local debug_msg = ''
	local OSclock = timer.getTime()
	TITS.avgTrackTic = (TITS.avgTrackTic + (OSclock-TITS.lastOSclock))/2
	TITS.lastOSclock =  OSclock 
	
	 local _status,_weaponData, _weaponVel =  pcall(function()
									return {params.weapon:getPoint(),params.weapon:getVelocity()}
									end)
	if _status then
		-- weapon still in fly, update last point
		params.weaponLastPoint = _weaponData[1]
		params.weaponLastVelocity =  _weaponData[2]
		-- get postion of alive tracked units
		--for t,v in pairs(TITS.TargetList) do
		--	if v.type == 'unit' then
		--		local u =  Unit.getByName(v.name)
		--		if u then
		--			local r = v
		--			r.lastPos = u:getPoint()
		--			r.lastVel = u:getVelocity()
		--			TITS.TargetList[t] = r
		--		end -- if u then
		--	end -- if v.type == 'unit' then
		--end --for t,v in pairs(TITS.TargetList) do
		return timer.getTime() + TITS.WeaponTrackingInterval -- keep tracking		
	else -- weapon hit
		local target 
		local targetPos
		--local targetPosOrg --LAB
		local tempTarget 
		local tempDist = nil
		local tempPos
		local correctedWeaponPos 
		local corrected = ''
		
		-- weapon had no valid target so look for the closest to impact
		for t,v in pairs(TITS.TargetList) do
			if v.impact then -- impact tracking target
				local oTarget
				local oTargetPos
				local oTargetPosOrg-- LAB
				local ignore = true
				local targetVel = {}

				--trigger.action.outText('name:'..v.name..'  type:'..v.type, 5 )-- LAB
				if v.type == 'unit' then
					oTarget = Unit.getByName(v.name)
					if oTarget then
						oTargetPos = oTarget:getPoint()
						--oTargetPosOrg = oTargetPos -- LAB
						targetVel = oTarget:getVelocity()
						oTargetPos = mist.vec.add(oTargetPos , mist.vec.scalar_mult(targetVel ,-1*TITS.avgTrackTic ) )
						ignore = false
						debug_msg = string.format('TITS: checking unit (%s) %s life:%s',v.name,v.displayName, oTarget:getLife())
					--else -- unit is dead
					--	oTargetPos = v.lastPos
					--	targetVel = v.lastVel
					--	oTargetPos = mist.vec.add(oTargetPos , mist.vec.scalar_mult(targetVel ,-1*TITS.avgTrackTic ) )
					--	debug_msg = string.format('TITS: checking unit (%s) %s DEAD',v.name,v.displayName)
					--	ignore = false
					end
				else -- it's a zone
					oTarget = trigger.misc.getZone(v.name)
					targetVel = { x=0, y=0, z=0}
					if oTarget then
						local tmp2DPos = oTarget.point
						 debug_msg = string.format('TITS: checking zone (%s) %s ',v.name,v.displayName)
						-- adjust to 3D vector
						oTargetPos = mist.utils.makeVec3GL(tmp2DPos)
						ignore = false
					end
				end		-- if v.type == 'unit' then	
				if not ignore then
					correctedWeaponPos = mist.vec.add(params.weaponLastPoint ,mist.vec.scalar_mult(params.weaponLastVelocity , TITS.WeaponTrackingInterval  ))
					local deltaY = math.abs(correctedWeaponPos.y - oTargetPos.y)
					if params.weaponLastVelocity.y ~= 0 and deltaY <= TITS.maxDeltaY then
					
						--local debugmsg = string.format('Correcting %s (1)-> uWp(x,y,z): (%.4f , %.4f , %.4f)',v.name,correctedWeaponPos.x,correctedWeaponPos.y,correctedWeaponPos.z )
						--TITS.messageToUnit(params.unitName,debugmsg,{},1)
						--debugmsg = string.format('Correcting %s (2)-> uTp(x,y,z): (%.4f , %.4f , %.4f)',v.name,oTargetPos.x,oTargetPos.y,oTargetPos.z )
						--TITS.messageToUnit(params.unitName,debugmsg,{},1)
						corrected = 'c'
						local flightTime = math.abs(deltaY / params.weaponLastVelocity.y)
						
						--debugmsg = string.format('Correcting %s (3)-> dY: %.4f  FT: %.4f    VS: %.4f ',v.name,deltaY,flightTime ,  params.weaponLastVelocity.y)
						--TITS.messageToUnit(params.unitName,debugmsg,{},1)
						
						correctedWeaponPos = mist.vec.add(correctedWeaponPos ,mist.vec.scalar_mult(params.weaponLastVelocity , flightTime ))
						
						--debugmsg = string.format('Correcting %s (4)-> cWp(x,y,z): (%.4f , %.4f , %.4f)',v.name,correctedWeaponPos.x,correctedWeaponPos.y,correctedWeaponPos.z )
						--TITS.messageToUnit(params.unitName,debugmsg,{},1)
						
						oTargetPos = mist.vec.add(oTargetPos ,mist.vec.scalar_mult(targetVel, flightTime ))

					end -- params.weaponLastVelocity.y ~= 0			
					local d =  math.floor(mist.utils.get3DDist(correctedWeaponPos,oTargetPos)) -- in meters
					local _slantRange = math.floor(mist.utils.get3DDist(params.launchPos,oTargetPos)*100)/100 -- in meters 
					debug_msg = string.format('%s distance: %s',debug_msg,d)
					--local debugmsg = string.format('%s -> d: %.4f m',v.name,d)
					--TITS.messageToUnit(params.unitName,debugmsg,{},1)
					
					if TITS.StoreData then
						local _tmpDir = TITS.getRelativeDirection(0 ,correctedWeaponPos,oTargetPos)
						local r = {
									weaponID = params.weaponID 
								,	weaponType = params.weaponName 
								,	target = v.name
								,   targetType = v.type
								, 	impactDistance = math.floor(d)
								, 	targetBearing = _tmpDir
								,   releaseHeading = params.releaseHeading
								,	releaseAlt = params.releaseAlt
								,	releaseSpeed = params.releaseSpeed
								, 	releasePitch = params.releasePitch
								, 	releaseRoll = params.releaseRoll
								, 	releaseYaw = params.releaseYaw
								, 	slantRange = _slantRange	
							}
							table.insert(TITS.dataStore_distance[params.unitName],r)
					end -- if TITS.StoreData then
					if v.type == 'zone' then	
						if v.bda and d <= oTarget.radius then -- See if the CIRCULAR zone was hit
							-- record the target that got splashed by the weapon BDA
							local r = {
											weaponID = params.weaponID 
										,	weaponType = params.weaponName 
										,	Target = v.name
									}				
							table.insert(TITS.reportWeaponsHits[params.unitName],r)
							if TITS.StoreData then										
								local r = {
										weaponID = params.weaponID 
									,	weaponType = params.weaponName 
									,	Target = v.name
									, 	Count = 1
								}	
								table.insert(TITS.dataStore_hits[params.unitName],r)
							end -- if TITS.StoreData then
						end -- 	if v.bda and d <= oTarget.radius then
					end -- if v.type = 'zone' then	
					
					if  tempDist then
						if d < tempDist then						   
							tempTarget = v.name
							tempPos = oTargetPos
							--tempPosOrg = oTargetPosOrg -- LAB
							tempDist = d
						end -- if d < tempDist then
					else -- if  tempDist then
						
						tempTarget = v.name
						tempPos = oTargetPos
						--tempPosOrg = oTargetPosOrg -- LAB
						tempDist = d
					end -- if not tempDist then
					--trigger.action.outText('closest :'..tempTarget..'    d:'..tempDist, 5 )-- LAB
				end  -- if not ignore then
			end --if v.impact then -- impact tracking target
			env.info(debug_msg,false)
		end -- for t,v in pairs(TITS.TargetList) do
		target = tempTarget
		targetPos = tempPos
		
		local _impactDistance = math.floor(tempDist*100)/100 -- in meters		

		local _direction = TITS.getRelativeDirection(params.releaseHeading ,targetPos,correctedWeaponPos)
		_direction = TITS.getClockDirection(_direction)
		
		local _slantRange = math.floor(mist.utils.get3DDist(params.launchPos,targetPos)*100)/100
		local _MOA = _slantRange * 0.029 / 100 -- Size of 1 MOA at slant range in meters (1 MOA @ 100m = 0.029m)
		local _impactDistanceMOA = math.floor((_impactDistance/ _MOA)*100)/100
		
		debug_msg = string.format('TITS: closest target (%s) distance:%s',target, _impactDistance)
		env.info(debug_msg,false)
		local r = {
						weaponID = params.weaponID 
					,	weaponType = params.weaponName 
					,	corrected = corrected 
					,	closestTarget = target
					, 	impactDistance = math.floor(_impactDistance)
					, 	impactDirection = _direction
					,	releaseHeading = params.releaseHeading
					,	releaseAlt = params.releaseAlt
					,	releaseSpeed = params.releaseSpeed
					, 	releasePitch = params.releasePitch
					, 	releaseRoll = params.releaseRoll
					, 	releaseYaw = params.releaseYaw
					, 	slantRange = _slantRange
					,	impactDistanceMOA = _impactDistanceMOA
					
		}
		--if params.weaponHasTarget then
		--	r.intendedTarget = params.target:getName()
		--end
		table.insert(TITS.reportWeaponsDistance[params.unitName],r)
		return nil
	end
end
--############################################################################################################################################
function TITS.convertSpeed(_unitName,_speed)
	local s
	local u
	 -- 1 = Knots, 2 = Mph, 3 = Kph	
	 -- _speed in m/s
	if TITS.SpeedUOM[_unitName] == 1 then
		s = _speed * 1.943844
		u = 'kt'
	elseif TITS.SpeedUOM[_unitName] == 2 then
		s = _speed * 2.23694
		u = 'mph'
	else
		s = _speed * 36
		u = 'kph'
	end
	return s,u
end
--############################################################################################################################################
function TITS.convertDistance(_unitName,_distance)
	local s
	local u
	-- 1 = feet, 2 = yards, 3 = meters
	-- _distance in meters
	if TITS.DistanceUOM[_unitName] == 1 then
		s = _distance * 3.28084
		u = 'ft'
	elseif TITS.DistanceUOM[_unitName] == 2 then
		s = _distance * 1.093613
		u = 'yd'
	else
		s = _distance 
		u = 'm'
	end
	return s,u
end
--############################################################################################################################################
function TITS.convertAltitude(_unitName,_altitude)
	local s
	local u
	-- 1 = feet, 2 = yards, 3 = meters
	-- _distance in meters
	if TITS.AltitudeUOM[_unitName] == 1 then
		s = _altitude * 3.28084
		u = 'ft'
	else
		s = _altitude 
		u = 'm'
	end
	return s,u
end
--############################################################################################################################################
function TITS.getRelativeDirection(_refHeading,_point1, _point2)
    local tgtBearing
    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z
    
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    
    local tgtAngle = math.deg(math.atan(yDiff/xDiff))
    
    if xDiff > 0 then 
    tgtBearing = 180 + tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle > 0 then
		tgtBearing = tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle < 0 then
		tgtBearing = 360 + tgtAngle
    end   
	
	tgtBearing = tgtBearing - _refHeading
	if tgtBearing > 360 then
		tgtBearing = tgtBearing - 360
	end
	if tgtBearing < 0 then
		tgtBearing =  360 + tgtBearing 
	end
	
    return tgtBearing
end
--############################################################################################################################################
function TITS.getClockDirection(_direction)
	-- by cfrag
	if not _direction then return 0 end
	while _direction < 0 do 
		_direction = _direction + 360
	end
		
	if _direction < 15 then -- special case 12 o'clock past 12 o'clock
		return 12
	end
	
	_direction = _direction + 15 -- add offset so we get all other times correct
	return math.floor(_direction/30)
	
end
--############################################################################################################################################
function TITS.getDirection(_point1, _point2)
    local tgtBearing
    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z
    
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    
    local tgtAngle = math.deg(math.atan(yDiff/xDiff))
    
    if xDiff > 0 then 
    tgtBearing = 180 + tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle > 0 then
    tgtBearing = tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle < 0 then
    tgtBearing = 360 + tgtAngle
    end
    
   
    return tgtBearing
end
--############################################################################################################################################
function TITS.MarkTargetWP(_unitName)
	local _unit = Unit.getByName(_unitName)
	-- count alive units
	local _numberOfUnits = 0

	local markableTargets = TITS.getWPmarkableTargets()
	for _,_targetZone in pairs(markableTargets) do	

			_numberOfUnits = _numberOfUnits + 1			
	
	end -- for _,_targetZone in pairs(range.bombingTargets) do	
	
	
	if _numberOfUnits > 0 then
		local r = mist.random(1,_numberOfUnits)

		-- get the random unit name

		local _targetName = markableTargets[r]
		local target 
		local targetPos
		local oTarget = TITS.getTarget(_targetName)
			
		if oTarget.type == 'unit' then
			target = Unit.getByName(_targetName)
			targetPos = target:getPoint()
		else
			target = trigger.misc.getZone(_targetName)
			targetPos = target.point
		end	


		-- mark the unit 
		local newPoint = mist.getRandPointInCircle(targetPos,TITS.MaxMarkDistance,50,0,360)
		local _alt = land.getHeight(newPoint)
		local smokePos = { x = newPoint.x, y = _alt, z = newPoint.y }
		local _dir = mist.utils.toDegree(mist.utils.getDir(mist.vec.sub(targetPos, smokePos)))
		local _dist = mist.utils.get2DDist(smokePos,targetPos)
		--local _type = target:getTypeName()

		trigger.action.smoke(smokePos, trigger.smokeColor.White )

		_direction = ''

		if _dir >= 0 and _dir <  22 then
				_direction = 'North'
			elseif _dir >= 22 and _dir <  68 then
				_direction = 'North East'
			elseif _dir >= 68 and _dir <  113 then
				_direction = 'East'
			elseif _dir >= 113 and _dir <  158 then
				_direction = 'South East'
			elseif _dir >= 158 and _dir <  203 then
				_direction = 'South'
			elseif _dir >= 203 and _dir <  248 then
				_direction = 'South West'
			elseif _dir >= 248 and _dir <  293 then
				_direction = 'West'
			elseif _dir >= 293 and _dir <  338 then
				_direction = 'North West'
			elseif _dir >= 338 and _dir <=  360 then
				_direction = 'North'
		end
		
		local _uom 
		_dist, _uom = TITS.convertDistance(_unitName,_dist)

		if _uom == 'ft' then 
			_dist = math.floor(_dist / 25) * 25
		else
			_dist = math.floor(_dist / 10) * 10
		end
		
		local MarkMessage = string.format("%s marked with WP\n%i %s, %s from the mark",oTarget.displayName, _dist,_uom, _direction)	
		TITS.LastDesignationMsg[_unitName] = MarkMessage
		TITS.messageToUnit(_unitName,MarkMessage,{},45)
		TITS.designatedTargetName = _targetName

	else
		TITS.messageToUnit(_unitName, "There is no targets available",{})
		TITS.LastDesignationMsg[_unitName] = 'no targets designated'
	end -- if _numberOfUnits > 0 then
	
end
--############################################################################################################################################
function TITS.getWPmarkableTargets()
	local _list={} -- target names
	for t,v in pairs(TITS.TargetList) do
		if v.des_wp  and TITS.IsAlive(v.name)  then
			table.insert(_list,v.name)
		end
	end
	return _list
end
--############################################################################################################################################
function TITS.RepeatLastDesMSG(_unitName)	
	TITS.messageToUnit(_unitName,TITS.LastDesignationMsg[_unitName] ,{},45)
	return true
end
--############################################################################################################################################
function TITS.showTargetsCoordinates(_unitName)
	local _msg = ''
	local _unit = Unit.getByName(_unitName)
	local pos
	ListTargetsCoordinates = TITS.getListCoordinatesTargets()
	for _,t in pairs(ListTargetsCoordinates) do

		local _target = Unit.getByName(t)
		if  _target then
			pos = _target:getPosition().p
		else
			_target = trigger.misc.getZone(t)
			pos = _target.point
		end
		
		local _coord = ''
		local lat, lon, alt = coord.LOtoLL(pos)
		if TITS.CoordFormat[_unitName] == 1 then --MGRS
			_coord = mist.tostringMGRS(coord.LLtoMGRS(coord.LOtoLL(pos)), 5)
		elseif  TITS.CoordFormat[_unitName] == 2 then -- DMS
			_coord = mist.tostringLL(lat, lon, 0, true)	
		else -- DM.mm
			_coord = mist.tostringLL(lat, lon, 2)
		end	
		local tt= TITS.getTarget(t)

		local a,auom = TITS.convertAltitude(_unitName,alt)
		--_msg = _msg..'['..tt.displayName..'] > '.._coord..'\n'
		_msg = _msg..string.format('[%s]->%s  (%i%s MSL)\n',tt.displayName,_coord,a,auom)
	end
	
	-- get coors of aim-off
	if TITS.OffsetMarkerName ~= '' then
		local aimoff =  Unit.getByName(TITS.OffsetMarkerName )
		if aimoff then
			pos = aimoff:getPosition().p
			if TITS.CoordFormat[_unitName] == 1 then --MGRS
				_coord = mist.tostringMGRS(coord.LLtoMGRS(coord.LOtoLL(pos)), 5)
			elseif  TITS.CoordFormat[_unitName] == 2 then -- DMS
				local lat, lon, alt = coord.LOtoLL(pos)
				_coord = mist.tostringLL(lat, lon, 0, true)				
			else -- DM.mm
				local lat, lon, alt = coord.LOtoLL(pos)
				_coord = mist.tostringLL(lat, lon, 2)
			end	
		
			_msg = string.format("Offset Marker: %s\n%s",_coord,_msg)
		end
	end
	if _msg ~= '' then
		TITS.messageToUnit(_unitName, _msg,{},30)
	end
    return true
end
--############################################################################################################################################
function TITS.getListCoordinatesTargets()
	local _list={} -- target names
	for t,v in pairs(TITS.TargetList) do
		if v.list_coor  and TITS.IsAlive(v.name) then
			table.insert(_list,v.name)
		end
	end
	return _list
end
--############################################################################################################################################
function TITS.OffsetDesignation(_unitName)
	local _unit = Unit.getByName(_unitName)
	-- count alive units
	local _numberOfUnits = 0

	local markableTargets = TITS.getOffSetDesTargets()
	for _,_targetZone in pairs(markableTargets) do	
			_numberOfUnits = _numberOfUnits + 1			
	end -- for _,_targetZone in pairs(range.bombingTargets) do	
	
	
	if _numberOfUnits > 0 then
		local r = mist.random(1,_numberOfUnits)

		-- get the random unit name

		local _targetName = markableTargets[r]
		local target 
		local targetPos
		local oTarget = TITS.getTarget(_targetName)
			
			
		if oTarget.type == 'unit' then
			target = Unit.getByName(_targetName)
			targetPos = target:getPoint()
		else
			target = trigger.misc.getZone(_targetName)
			targetPos = target.point
		end	

		-- mark the unit 
		--local _type = target:getTypeName()
		local aimoff =  Unit.getByName(TITS.OffsetMarkerName)
		

		local _heading = mist.utils.getHeadingPoints(aimoff:getPoint(),targetPos,true)
		 _heading =  math.floor(mist.utils.toDegree(_heading))
		local _dist = mist.utils.get3DDist(aimoff:getPoint(),targetPos)
		
		local _uom 
		_dist, _uom = TITS.convertDistance(_unitName,_dist)
		_dist = math.floor(_dist / 5) * 5
		
		local MarkMessage = string.format("Target: %s  %i %s @ %s° (true)  from the Offset markert",oTarget.displayName, _dist,_uom,_heading )	
		TITS.LastDesignationMsg[_unitName] = MarkMessage
		TITS.messageToUnit(_unitName,MarkMessage,{},45)
		TITS.designatedTargetName = _targetName

	else
		TITS.messageToUnit(_unitName, "There is no targets available",{})
		TITS.LastDesignationMsg[_unitName] = 'no targets designated'
	end -- if _numberOfUnits > 0 then
	
end
--############################################################################################################################################
function TITS.getOffSetDesTargets()
	local _list={} -- target names
	for t,v in pairs(TITS.TargetList) do
		if v.des_aimoff  and TITS.IsAlive(v.name)  then
			table.insert(_list,v.name)
		end
	end
	return _list
end
--############################################################################################################################################
function TITS.LaserDesignation(_unitName)
	local _unit = Unit.getByName(_unitName)
--	TITS.laseUnit('Bunker  R2-28', 'JTAC-1', '1688')

	local _numberOfUnits = 0

	local markableTargets = TITS.getLaserDesTargets()
	for _,_targetZone in pairs(markableTargets) do	
			_numberOfUnits = _numberOfUnits + 1			
	end -- for _,_targetZone in pairs(range.bombingTargets) do	
	
	
	if _numberOfUnits > 0 then
		local r = mist.random(1,_numberOfUnits)

		-- get the random unit name

		local _targetName = markableTargets[r]
		local target 
		local targetPos
		local oTarget = TITS.getTarget(_targetName)
			
		if oTarget.type == 'unit' then
			target = Unit.getByName(_targetName)
			targetPos = target:getPoint()
		else
			target = trigger.misc.getZone(_targetName)
			targetPos = target.point
		end	

		-- mark the target 
		--local _type = target:getTypeName()
		local JTAC =  Unit.getByName(TITS.LaserDesignatorName)

		-- check if laser is in use
		if TITS.LaserSpot  ~= nil then
			Spot.destroy(TITS.LaserSpot )
			TITS.LaserSpot  = nil
		end
		if TITS.IRSpot  ~= nil then
			Spot.destroy(TITS.IRSpot )
			TITS.IRSpot  = nil
		end

		-- lase the unit
		local targetPosUpdated = { x = targetPos.x, y = targetPos.y + 2.0, z = targetPos.z }
		local spots = {}
		 local status, result = pcall(function()
            spots['irPoint'] = Spot.createInfraRed(JTAC, { x = 0, y = 2.0, z = 0 }, targetPos)
            spots['laserPoint'] = Spot.createLaser(JTAC, { x = 0, y = 2.0, z = 0 }, targetPosUpdated, TITS.LaserDesignatorCode)
            return spots
        end)

		local MarkMessage = ''
        if not status then
			MarkMessage = 'Laser error: cannot designate'
			trigger.action.outText('error',10)
            env.error('ERROR: ' .. assert(result), false)
        else
			
            if result.irPoint then
              TITS.IRSpot= result.irPoint --store so we can remove after
			  MarkMessage = 'IR'
            end
            if result.laserPoint then		
				if MarkMessage ~= '' then
					MarkMessage = MarkMessage..' and '
				end
				MarkMessage = MarkMessage..'Laser on '..TITS.LaserDesignatorCode
               TITS.LaserSpot  = result.laserPoint
            end
        end
		
		if MarkMessage ~= 'Laser error: cannot designate' then
			local _coord = mist.tostringMGRS(coord.LLtoMGRS(coord.LOtoLL(targetPos)), 5)
			--MarkMessage = string.format("Designating %s by %s\nCoordinates: %s",oTarget.displayName,  MarkMessage,_coord )	
			MarkMessage = string.format("Designating %s by %s",oTarget.displayName,MarkMessage)	
		end
		TITS.LastDesignationMsg[_unitName] = MarkMessage
		TITS.messageToUnit(_unitName,MarkMessage,{},45)
		TITS.designatedTargetName = _targetName

	else
		TITS.messageToUnit(_unitName, "There is no targets available",{})
		TITS.LastDesignationMsg[_unitName] = 'no targets designated'
	end -- if _numberOfUnits > 0 then

end
--############################################################################################################################################
function TITS.getLaserDesTargets()
	local _list={} -- target names
	for t,v in pairs(TITS.TargetList) do
		if v.des_laser and TITS.IsAlive(v.name) then
			table.insert(_list,v.name)
		end
	end
	return _list
end
--############################################################################################################################################
function TITS.CancelLaserDes(_unitName)
	-- check if laser is in use
		if TITS.LaserSpot  ~= nil then
			Spot.destroy(TITS.LaserSpot )
			TITS.LaserSpot  = nil
		end
		if TITS.IRSpot  ~= nil then
			Spot.destroy(TITS.IRSpot )
			TITS.IRSpot  = nil
		end
		
		if TITS.designatedTargetName ~= '' then
			TITS.messageToUnit(_unitName,'Roger. Target undesignated ',{})
		end 
		
		--if  string.find(TITS.LastDesignationMsg[_unitName] , "Designating") then
		TITS.LastDesignationMsg[_unitName] = ''
		TITS.designatedTargetName = '' 
		--end
		
	return true
end
--############################################################################################################################################
function TITS.AddTarget(_TargetName,_DisplayName,_Type,_TrackImpact,_TrackStrafing,_ReportHits,_AllowOffsetDesignation,_AllowSmokeDesignation,_AllowLaserDesignation,_AllowListCoordinates)
	local dn = _DisplayName
	
	if dn == '' then
		dn = _TargetName
	end
	
	table.insert(TITS.TargetList,
								{	
									name	 	= 	_TargetName,
									displayName = 	dn,
									type 		= 	_Type, 
									impact 		=	_TrackImpact,	
									strafing 	= 	_TrackStrafing, 	
									bda 		= 	_ReportHits,		
									des_aimoff 	= 	_AllowOffsetDesignation,	
									des_wp 		= 	_AllowSmokeDesignation,	
									des_laser 	= 	_AllowLaserDesignation,	
									list_coor 	= 	_AllowListCoordinates,
									lastPos		=	{},	
									lastVel		=	{}
									
									}
				)
	return true
end
--############################################################################################################################################
function TITS.IsAlive(_name)
	local isAlive = true
	
	t = TITS.getTarget(_name)
	
	if t.type == 'unit' then
		local _unit = Unit.getByName(_name)
		if _unit then
			if _unit:getLife() < 1 then
				isAlive = false
			end
		else
			isAlive = false
		end
	end
	return isAlive
end
--############################################################################################################################################
function TITS.storeStrafingHits(_unitName)

	local unitTargetList = TITS.strafingHitCounter[_unitName]
	if unitTargetList  ~= nil then
		for t,shellList in pairs(unitTargetList) do
			if shellList ~= nil  then						
				for s,count in pairs(shellList) do
					local r = {
							weaponID = -1   --(-1 if it is a shell weapon)
						,	weaponType = s
						,	Target = t
						, 	Count  = count
					}
					table.insert(TITS.dataStore_hits[_unitName],r)
				end -- for s,count in pairs(shellList)
			end -- if shellList > 0 then
		end -- for t,shellList in pairs(unitTargetList) do
	end
	return true
end -- function
--############################################################################################################################################
function TITS.exportTrackingData()
--[[
				TITS.dataStore_distance[params.unitName]				
				{
						weaponID 
					,	weaponType 
					,	target 
					,   targetType 
					, 	impactDistance 
					, 	targetBearing 
					,   releaseHeading 
					,	releaseAlt 
					,	releaseSpeed 
					, 	releasePitch 
					, 	releaseRoll 
					, 	releaseYaw 
					, 	slantRange
				}
	
								
				TITS.dataStore_hits[params.unitName]
				{
						weaponID    (-1 if it is a shell weapon)
					,	weaponType 
					,	Target 
					, 	Count 
				}	
				
				exported Data (metric units)
				{
						unitName
					,	targetName
					,	targetType
					,	targetDisplayName
					,	targetHitCount
					,	impactDistance      (0 if it is a shell weapon) 
					,	targetBearing 
					,	weaponID    		(-1 if it is a shell weapon)
					,   weaponType 
					,   releaseHeading 		(0 if it is a shell weapon) 
					,	releaseAlt 			(0 if it is a shell weapon) 
					,	releaseSpeed 		(0 if it is a shell weapon) 
					, 	releasePitch 		(0 if it is a shell weapon) 
					, 	releaseRoll 		(0 if it is a shell weapon) 
					, 	releaseYaw 			(0 if it is a shell weapon) 
					, 	slantRange			(0 if it is a shell weapon) 
				}
				
]]--

	local DataSet = {}
	
	-- Process shell weapons 

	for u,TT in pairs(TITS.dataStore_hits) do
		for _,r in pairs(TT) do
			if r.weaponID == -1 then
				local t = TITS.getTarget(r.Target)	
				local newRow = {
								unitName 			= u
							,	targetName 			= r.Target
							,	targetType 			= t.type
							,	targetDisplayName 	= t.displayName
							,	targetHitCount 		= r.Count
							,	impactDistance     	= 0	--(0 if it is a shell weapon) 
							,	targetBearing 		= 0 --(0 if it is a shell weapon) 
							,	weaponID    		= -1 --(-1 if it is a shell weapon)
							,   weaponType 			= r.weaponType
							,   releaseHeading 		= 0 --(0 if it is a shell weapon) 
							,	releaseAlt 			= 0 --(0 if it is a shell weapon) 
							,	releaseSpeed 		= 0 --(0 if it is a shell weapon) 
							, 	releasePitch 		= 0 --(0 if it is a shell weapon) 
							, 	releaseRoll 		= 0 --(0 if it is a shell weapon) 
							, 	releaseYaw 			= 0 --(0 if it is a shell weapon) 
							, 	slantRange			= 0 --(0 if it is a shell weapon) 
				}
				table.insert(DataSet, newRow)
			end --if r.weaponID == -1 then 
		end -- for _,r in pairs(TT) do
	end -- for u,r in pairs(TITS.dataStore_hits) do
	-- process all others
	for u,TT in pairs(TITS.dataStore_distance) do
		for _,r in pairs(TT) do
			
			local t = TITS.getTarget(r.target)	
			
		
			-- find hits
			local hits = TITS.dataStore_hits[u]
			local hitCount = 0
			
			for _,v in pairs(hits) do
				if 	v.weaponID == r.weaponID and v.weaponType == r.weaponType and v.Target == r.target then
					hitCount = 1
					break
				end
			end -- for _,v in pairs(hits) do
			
			-- created the record
			local newRow = {
							unitName 			= u
						,	targetName 			= r.target
						,	targetType 			= t.type
						,	targetDisplayName 	= t.displayName
						,	targetHitCount 		= hitCount
						,	impactDistance     	= r.impactDistance	--(0 if it is a shell weapon) 
						,	targetBearing 		= r.targetBearing --(0 if it is a shell weapon) 
						,	weaponID    		= r.weaponID --(-1 if it is a shell weapon)
						,   weaponType 			= r.weaponType
						,   releaseHeading 		= r.releaseHeading --(0 if it is a shell weapon) 
						,	releaseAlt 			= r.releaseAlt --(0 if it is a shell weapon) 
						,	releaseSpeed 		= r.releaseSpeed --(0 if it is a shell weapon) 
						, 	releasePitch 		= r.releasePitch --(0 if it is a shell weapon) 
						, 	releaseRoll 		= r.releaseRoll --(0 if it is a shell weapon) 
						, 	releaseYaw 			= r.releaseYaw --(0 if it is a shell weapon) 
						, 	slantRange			= r.slantRange --(0 if it is a shell weapon) 
			}
			table.insert(DataSet, newRow)
	
		end -- for _,r in pairs(TT) do
	end -- for u,r in pairs(TITS.dataStore_hits) do
	
	
	return DataSet
end
--############################################################################################################################################
function TITS.resetTrackingData()

	for u,_ in pairs(TITS.dataStore_distance) do
		TITS.dataStore_distance[u] = {}
	end

	for u,_ in pairs(TITS.dataStore_hits) do
		TITS.dataStore_hits[u] = {}
	end
	
	return true
end
--############################################################################################################################################
function TITS.startAttack(_unitName)
	TITS.ReportAttack({_unitName,'in'})
	return true
end
--############################################################################################################################################
function TITS.endAttack(_unitName)
	TITS.ReportAttack({_unitName,'off'})
	return true
end
--############################################################################################################################################
function TITS.getTrackedUnitNames()
	local rt = {}
	
	for k,_ in pairs(TITS.attackingUnits[unitName]) do
		table.insert(rt,k)
	end
	return rt
end
--############################################################################################################################################


-- Main Body
do
	world.addEventHandler(TITS.eventHandler)
end